/*
  fragrouter.c

  network IDS evasion toolkit.

  Dug Song <dugsong@anzen.com>

  Copyright (c) 1999 Anzen Computing. All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:
  
  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
  2. Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
  3. All advertising materials mentioning features or use of this software
     must display the following acknowledgement:
     This product includes software developed by Anzen Computing.
  4. Neither the name of Anzen Computing nor the names of its
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
  IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  
  $Id: fragrouter.c,v 1.16 1999/05/13 15:10:12 dugsong Exp $
*/

#include "config.h"

#ifdef STDC_HEADERS
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif
#ifdef HAVE_UNISTD_H
#include <sys/types.h>
#include <unistd.h>
#endif
#include "frag.h"
#include "sniff.h"
#include "send.h"
#include "attack.h"

#define FRAGROUTER_VERSION	"1.0"

void
usage(void)
{
  fprintf(stderr, "Version " FRAGROUTER_VERSION "\n"
      "Usage: fragrouter ATTACK [-i interface] [-p] <host>\n\n"
      " where ATTACK is one of the following:\n\n"
      " -B1: base-1: normal IP forwarding\n"
      " -F1: frag-1: ordered 8-byte IP fragments\n"
      "   2: frag-2: ordered 24-byte IP fragments\n"
      "   3: frag-3: ordered 8-byte IP fragments, one out of order\n"
      "   4: frag-4: ordered 8-byte IP fragments, one duplicate\n"
      "   5: frag-5: out of order 8-byte fragments, one duplicate\n"
      "   6: frag-6: ordered 8-byte fragments, marked last frag first\n"
      "   7: frag-7: ordered 16-byte fragments, fwd-overwriting\n"
#if 0
      " -T1: tcp-1:  3-way handshake, fake FIN/RST, 1 byte packets\n"
      "   2: tcp-2:  3-way handshake, 1 byte packets with seqnum wrap\n"
      "   3: tcp-3:  3-way handshake, 1 byte packets, one duplicate\n"
      "   4: tcp-4:  3-way handshake, 1 byte packets, one overwriting\n"
      "   5: tcp-5:  3-way handshake, 1-byte packets, 2-byte fwd-overwrite\n"
      "   6: tcp-6:  3-way handshake, 1-byte packets, invalid seqnum jump\n"
      "   7: tcp-7:  3-way handshake, 1-byte packets, mixed bad seqnum\n"
      "   8: tcp-8:  3-way handshake, 1-byte packets, one out-of-order\n"
      "   9: tcp-9:  3-way handshake, 1-byte packets, out-of-order\n"
      " -C1: tcbc-1: no handshake, 1-byte packets\n"
      "   2: tcbc-2: 3-way handshake, 1-byte packets, interleaved SYNs\n"
      "   3: tcbc-3: no handshake, 1-byte packets, then real connection\n"
      " -R1: tcbt-1: 3-way handshake, RST, 3-way handshake, 1-byte packets\n"
      "   2: tcbt-2: 3-way handshake, 1-byte packets, RST, 1-byte packets\n"
      " -I1: ins-1:  3-way handshake, 1-byte packets, bad IP checksum\n"
      "   2: ins-2:  3-way handshake, 1-byte packets, bad TCP checksum\n"
      "   3: ins-3:  3-way handshake, 1-byte packets, no ACK set\n"
      " -E1: evad-1: 3-way handshake, data in SYN"
#endif
      "\n");
  exit(1);
}

int
main(int argc, char *argv[])
{
  char c, ebuf[BUFSIZ], *dev = NULL;
  int num = 0, type = -1;
  void *attack;
  
  while ((c = getopt(argc, argv, "B:F:T:C:R:I:E:i:pVh")) != EOF) {
    switch (c) {
    case 'B':
      type = ATTACK_BASELINE;
      num = atoi(optarg);
      break;
    case 'F':
      type = ATTACK_FRAG;
      num = atoi(optarg);
      break;
    case 'T':
      type = ATTACK_TCP;
      num = atoi(optarg);
      break;
    case 'C':
      type = ATTACK_CONNECT;
      num = atoi(optarg);
      break;
    case 'R':
      type = ATTACK_TEARDOWN;
      num = atoi(optarg);
      break;
    case 'I':
      type = ATTACK_INSERT;
      num = atoi(optarg);
      break;
    case 'E':
      type = ATTACK_EVADE;
      num = atoi(optarg);
      break;
    case 'p':
      frag_init(1);
      break;
    case 'i':
      dev = strdup(optarg);
      break;
    default:
      usage();
    }
  }
  argc -= optind;
  argv += optind;
  
  if (argc != 0 || type == -1) usage();

#if (__linux__)
  /* Linux doesn't let us forge packets with short protocol headers. feh. */
  frag_init(1);
#endif
  
  if (!sniff_init(dev, ebuf)) {
    fprintf(stderr, "fragrouter: sniff_init failed: %s\n", ebuf);
    exit(1);
  }
  if (!send_init(ebuf)) {
    fprintf(stderr, "fragrouter: send_init failed: %s\n", ebuf);
    exit(1);
  }
  if (!(attack = attack_init(type, num, ebuf))) {
    fprintf(stderr, "fragrouter: attack_init failed: %s\n", ebuf);
    exit(1);
  }
  fprintf(stderr, "fragrouter: %s-%d started\n", attack_name(type), num);
  
  sniff_loop(attack);

  exit(0);
}
